@@ -39,7 +39,7 @@
 
 typedef struct
 {
-  GList                *task_lists;
+  GHashTable           *task_lists;
 
   ESourceRegistry      *source_registry;
   ECredentialsPrompter *credentials_prompter;
@@ -67,6 +67,7 @@ enum
   PROP_ICON,
   PROP_ID,
   PROP_NAME,
+  PROP_PROVIDER_TYPE,
   PROP_REGISTRY,
   N_PROPS
 };
@@ -140,6 +141,7 @@ on_client_connected_cb (GObject      *source_object,
                                       error->message,
                                       NULL,
                                       NULL);
+      gtd_object_pop_loading (GTD_OBJECT (self));
       return;
     }
 
@@ -151,7 +153,7 @@ on_client_connected_cb (GObject      *source_object,
   /* creates a new task list */
   list = gtd_task_list_eds_new (GTD_PROVIDER (self), source, client);
 
-  priv->task_lists = g_list_append (priv->task_lists, list);
+  g_hash_table_add (priv->task_lists, list);
 
   g_object_set_data (G_OBJECT (source), "task-list", list);
 
@@ -179,6 +181,7 @@ on_source_added_cb (GtdProviderEds *provider,
    * The pop_loading() is actually emited by GtdTaskListEds, after the
    * ECalClientView sends the :complete signal.
    */
+  gtd_object_push_loading (GTD_OBJECT (provider));
   gtd_object_push_loading (GTD_OBJECT (gtd_manager_get_default ()));
 
   e_cal_client_connect (source,
@@ -196,10 +199,15 @@ on_source_removed_cb (GtdProviderEds *provider,
   GtdProviderEdsPrivate *priv;
   GtdTaskList *list;
 
+  GTD_ENTRY;
+
   priv = gtd_provider_eds_get_instance_private (provider);
   list = g_object_get_data (G_OBJECT (source), "task-list");
 
-  priv->task_lists = g_list_remove (priv->task_lists, list);
+  if (!g_hash_table_contains (priv->task_lists, list))
+    GTD_RETURN ();
+
+  g_hash_table_remove (priv->task_lists, list);
 
   /*
    * Since all subclasses will have this signal given that they
@@ -207,6 +215,8 @@ on_source_removed_cb (GtdProviderEds *provider,
    * to let it stay here.
    */
   g_signal_emit_by_name (provider, "list-removed", list);
+
+  GTD_EXIT;
 }
 
 static void
@@ -466,7 +476,11 @@ on_task_list_removed_cb (ESource      *source,
 
   REPORT_ERROR (_("An error occurred while modifying a task list"), error);
 
-  g_signal_emit_by_name (self, "list-removed", list);
+  /*
+   * Note: we don't need to emit the "list-removed" signal here
+   * because EDS will emit the "source-removed" signal, and we
+   * emit "list-removed" there.
+   */
 
   GTD_EXIT;
 }
@@ -492,6 +506,14 @@ gtd_provider_eds_get_name (GtdProvider *provider)
   return GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->get_name (GTD_PROVIDER_EDS (provider));
 }
 
+static const gchar*
+gtd_provider_eds_get_provider_type (GtdProvider *provider)
+{
+  g_return_val_if_fail (GTD_IS_PROVIDER_EDS (provider), NULL);
+
+  return GTD_PROVIDER_EDS_CLASS (G_OBJECT_GET_CLASS (provider))->get_provider_type (GTD_PROVIDER_EDS (provider));
+}
+
 static const gchar*
 gtd_provider_eds_get_description (GtdProvider *provider)
 {
@@ -716,7 +738,7 @@ gtd_provider_eds_get_task_lists (GtdProvider *provider)
 {
   GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (GTD_PROVIDER_EDS (provider));
 
-  return priv->task_lists;
+  return g_hash_table_get_keys (priv->task_lists);
 }
 
 static GtdTaskList*
@@ -763,6 +785,7 @@ gtd_provider_iface_init (GtdProviderInterface *iface)
 {
   iface->get_id = gtd_provider_eds_get_id;
   iface->get_name = gtd_provider_eds_get_name;
+  iface->get_provider_type = gtd_provider_eds_get_provider_type;
   iface->get_description = gtd_provider_eds_get_description;
   iface->get_enabled = gtd_provider_eds_get_enabled;
   iface->get_icon = gtd_provider_eds_get_icon;
@@ -795,6 +818,7 @@ gtd_provider_eds_finalize (GObject *object)
   g_clear_object (&priv->cancellable);
   g_clear_object (&priv->credentials_prompter);
   g_clear_object (&priv->source_registry);
+  g_clear_pointer (&priv->task_lists, g_hash_table_destroy);
 
   G_OBJECT_CLASS (gtd_provider_eds_parent_class)->finalize (object);
 }
@@ -901,6 +925,10 @@ gtd_provider_eds_get_property (GObject    *object,
       g_value_set_string (value, gtd_provider_eds_get_name (provider));
       break;
 
+    case PROP_PROVIDER_TYPE:
+      g_value_set_string (value, gtd_provider_eds_get_provider_type (provider));
+      break;
+
     case PROP_REGISTRY:
       g_value_set_object (value, priv->source_registry);
       break;
@@ -951,6 +979,7 @@ gtd_provider_eds_class_init (GtdProviderEdsClass *klass)
   g_object_class_override_property (object_class, PROP_ICON, "icon");
   g_object_class_override_property (object_class, PROP_ID, "id");
   g_object_class_override_property (object_class, PROP_NAME, "name");
+  g_object_class_override_property (object_class, PROP_PROVIDER_TYPE, "provider-type");
 
   g_object_class_install_property (object_class,
                                    PROP_REGISTRY,
@@ -967,8 +996,7 @@ gtd_provider_eds_init (GtdProviderEds *self)
   GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
 
   priv->cancellable = g_cancellable_new ();
-
-  gtd_object_push_loading (GTD_OBJECT (self));
+  priv->task_lists = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
 
 GtdProviderEds*
