@@ -68,6 +68,7 @@ enum
   PROP_ICON,
   PROP_ID,
   PROP_NAME,
+  PROP_PROVIDER_TYPE,
   PROP_SOURCE,
   LAST_PROP
 };
@@ -83,6 +84,7 @@ print_task (GString *output,
 {
   GtdTaskList *list;
   GDateTime *dt;
+  const gchar *description;
   gint priority;
   gboolean is_complete;
 
@@ -90,6 +92,7 @@ print_task (GString *output,
   priority = gtd_task_get_priority (task);
   dt = gtd_task_get_due_date (task);
   list = gtd_task_get_list (task);
+  description = gtd_task_get_description (task);
 
   if (is_complete)
     g_string_append (output, "x ");
@@ -115,6 +118,12 @@ print_task (GString *output,
       g_string_append_printf (output, " due:%s", formatted_time);
     }
 
+  if (description)
+    {
+      g_autofree gchar *new_description = g_strescape (description, NULL);
+      g_string_append_printf (output, " note:\"%s\"", new_description);
+    }
+
   g_string_append (output, "\n");
 }
 
@@ -395,6 +404,12 @@ gtd_provider_todo_txt_get_name (GtdProvider *provider)
   return _("Todo.txt");
 }
 
+static const gchar*
+gtd_provider_todo_txt_get_provider_type (GtdProvider *provider)
+{
+  return "todo-txt";
+}
+
 static const gchar*
 gtd_provider_todo_txt_get_description (GtdProvider *provider)
 {
@@ -509,7 +524,7 @@ gtd_provider_todo_txt_get_task_lists (GtdProvider *provider)
 {
   GtdProviderTodoTxt *self = GTD_PROVIDER_TODO_TXT (provider);
 
-  return self->task_lists;
+  return g_list_copy (self->task_lists);
 }
 
 static GtdTaskList*
@@ -530,6 +545,7 @@ gtd_provider_iface_init (GtdProviderInterface *iface)
 {
   iface->get_id = gtd_provider_todo_txt_get_id;
   iface->get_name = gtd_provider_todo_txt_get_name;
+  iface->get_provider_type = gtd_provider_todo_txt_get_provider_type;
   iface->get_description = gtd_provider_todo_txt_get_description;
   iface->get_enabled = gtd_provider_todo_txt_get_enabled;
   iface->get_icon = gtd_provider_todo_txt_get_icon;
@@ -594,6 +610,10 @@ gtd_provider_todo_txt_get_property (GObject    *object,
       g_value_set_string (value, gtd_provider_todo_txt_get_name (provider));
       break;
 
+    case PROP_PROVIDER_TYPE:
+      g_value_set_string (value, gtd_provider_todo_txt_get_provider_type (provider));
+      break;
+
     case PROP_SOURCE:
       g_value_set_object (value, GTD_PROVIDER_TODO_TXT (provider)->source_file);
       break;
@@ -646,6 +666,7 @@ gtd_provider_todo_txt_class_init (GtdProviderTodoTxtClass *klass)
   g_object_class_override_property (object_class, PROP_ICON, "icon");
   g_object_class_override_property (object_class, PROP_ID, "id");
   g_object_class_override_property (object_class, PROP_NAME, "name");
+  g_object_class_override_property (object_class, PROP_PROVIDER_TYPE, "provider-type");
 }
 
 static void
