@@ -126,6 +126,7 @@ enum
   PROP_ICON,
   PROP_ID,
   PROP_NAME,
+  PROP_PROVIDER_TYPE,
   PROP_GOA_OBJECT,
   LAST_PROP
 };
@@ -1117,6 +1118,12 @@ gtd_provider_todoist_get_name (GtdProvider *provider)
   return _("Todoist");
 }
 
+static const gchar*
+gtd_provider_todoist_get_provider_type (GtdProvider *provider)
+{
+  return "todoist";
+}
+
 static const gchar*
 gtd_provider_todoist_get_description (GtdProvider *provider)
 {
@@ -1433,6 +1440,7 @@ gtd_provider_iface_init (GtdProviderInterface *iface)
 {
   iface->get_id = gtd_provider_todoist_get_id;
   iface->get_name = gtd_provider_todoist_get_name;
+  iface->get_provider_type = gtd_provider_todoist_get_provider_type;
   iface->get_description = gtd_provider_todoist_get_description;
   iface->get_enabled = gtd_provider_todoist_get_enabled;
   iface->get_icon = gtd_provider_todoist_get_icon;
@@ -1492,6 +1500,10 @@ gtd_provider_todoist_get_property (GObject    *object,
       g_value_set_string (value, gtd_provider_todoist_get_name (provider));
       break;
 
+    case PROP_PROVIDER_TYPE:
+      g_value_set_string (value, gtd_provider_todoist_get_provider_type (provider));
+      break;
+
     case PROP_GOA_OBJECT:
       break;
 
@@ -1550,6 +1562,7 @@ gtd_provider_todoist_class_init (GtdProviderTodoistClass *klass)
   g_object_class_override_property (object_class, PROP_ICON, "icon");
   g_object_class_override_property (object_class, PROP_ID, "id");
   g_object_class_override_property (object_class, PROP_NAME, "name");
+  g_object_class_override_property (object_class, PROP_PROVIDER_TYPE, "provider-type");
 }
 
 static void
