@@ -29,6 +29,7 @@
 #include "gtd-plugin-dialog.h"
 #include "gtd-vcs-identifier.h"
 #include "gtd-window.h"
+#include "gtd-window-private.h"
 
 #include <glib.h>
 #include <glib-object.h>
@@ -292,6 +293,13 @@ gtd_application_startup (GApplication *application)
   /* Load the plugins */
   gtd_manager_load_plugins (gtd_manager_get_default ());
 
+  /*
+   * This will select select the first panel of the sidebar, since
+   * at this point the panels are already loaded and the sidebar has
+   * all the rows set up.
+   */
+  _gtd_window_finish_startup (GTD_WINDOW (self->window));
+
   GTD_EXIT;
 }
 
