@@ -37,6 +37,9 @@ GtdTask*             gtd_edit_pane_get_task                      (GtdEditPane
 void                 gtd_edit_pane_set_task                      (GtdEditPane        *self,
                                                                   GtdTask            *task);
 
+void                 gtd_edit_pane_set_markdown_renderer         (GtdEditPane         *self,
+                                                                  GtdMarkdownRenderer *renderer);
+
 G_END_DECLS
 
 #endif /* GTD_EDIT_PANE_H */
