@@ -25,6 +25,7 @@
 #include "gtd-empty-list-widget.h"
 #include "gtd-task-list-view.h"
 #include "gtd-manager.h"
+#include "gtd-markdown-renderer.h"
 #include "gtd-new-task-row.h"
 #include "gtd-notification.h"
 #include "gtd-provider.h"
@@ -71,22 +72,25 @@ typedef struct
   GtkWidget             *empty_box;
   GtkListBox            *listbox;
   GtkListBoxRow         *new_task_row;
-  GtkRevealer           *revealer;
-  GtkImage              *done_image;
-  GtkLabel              *done_label;
-  GtkWidget             *viewport;
   GtkWidget             *scrolled_window;
+  GtkStack              *stack;
+  GtkWidget             *viewport;
 
   /* internal */
   gboolean               can_toggle;
-  gboolean               show_completed;
   gboolean               show_due_date;
   gboolean               show_list_name;
   gboolean               handle_subtasks;
-  GList                 *list;
+  GHashTable            *tasks;
   GtdTaskList           *task_list;
   GDateTime             *default_date;
 
+  guint                  idle_handler_id;
+  guint                  scroll_to_bottom_handler_id;
+
+  /* Markup renderer*/
+  GtdMarkdownRenderer   *renderer;
+
   /* DnD autoscroll */
   guint                  scroll_timeout_id;
   gboolean               scroll_up;
@@ -102,6 +106,10 @@ typedef struct
   GtdTaskListViewHeaderFunc header_func;
   gpointer                  header_user_data;
 
+  /* Custom filter function data */
+  GtdTaskListViewFilterFunc filter_func;
+  gpointer                  filter_user_data;
+
   /* Custom sorting function data */
   GtdTaskListViewSortFunc sort_func;
   gpointer                sort_user_data;
@@ -119,12 +127,32 @@ struct _GtdTaskListView
   GtdTaskListViewPrivate *priv;
 };
 
+typedef enum
+{
+  GTD_IDLE_STATE_STARTED,
+  GTD_IDLE_STATE_LOADING,
+  GTD_IDLE_STATE_COMPLETE,
+  GTD_IDLE_STATE_FINISHED,
+} GtdIdleState;
+
+typedef struct
+{
+  GtdTaskListView      *self;
+  GtdIdleState          state;
+  GPtrArray            *added;
+  GPtrArray            *removed;
+  guint32               current_item;
+} GtdIdleData;
+
 #define COLOR_TEMPLATE               "viewport {background-color: %s;}"
 #define DND_SCROLL_OFFSET            24 //px
 #define LUMINANCE(c)                 (0.299 * c->red + 0.587 * c->green + 0.114 * c->blue)
 #define TASK_REMOVED_NOTIFICATION_ID "task-removed-id"
+#define N_TASKS_BEFORE_LOAD          50
 
 
+static gboolean      idle_process_items_cb                       (gpointer            data);
+
 static void          on_clear_completed_tasks_activated_cb       (GSimpleAction      *simple,
                                                                   GVariant           *parameter,
                                                                   gpointer            user_data);
@@ -132,10 +160,6 @@ static void          on_clear_completed_tasks_activated_cb       (GSimpleAction
 static void          on_remove_task_row_cb                       (GtdTaskRow         *row,
                                                                   GtdTaskListView    *self);
 
-static void          on_task_completed_cb                        (GtdTask             *task,
-                                                                  GParamSpec          *pspec,
-                                                                  GtdTaskListView     *self);
-
 static void          on_task_list_task_added_cb                  (GtdTaskList         *list,
                                                                   GtdTask             *task,
                                                                   GtdTaskListView     *self);
@@ -149,6 +173,8 @@ static void          on_task_row_entered_cb                      (GtdTaskListVie
 static void          on_task_row_exited_cb                       (GtdTaskListView    *self,
                                                                   GtdTaskRow         *row);
 
+static gboolean      scroll_to_bottom_cb                         (gpointer            data);
+
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtdTaskListView, gtd_task_list_view, GTK_TYPE_OVERLAY)
 
@@ -166,7 +192,6 @@ enum {
   PROP_0,
   PROP_COLOR,
   PROP_HANDLE_SUBTASKS,
-  PROP_SHOW_COMPLETED,
   PROP_SHOW_LIST_NAME,
   PROP_SHOW_DUE_DATE,
   PROP_SHOW_NEW_TASK_ROW,
@@ -181,6 +206,41 @@ typedef gboolean     (*IterateSubtaskFunc)                       (GtdTaskListVie
  * Auxiliary methods
  */
 
+static void
+idle_data_free (gpointer data)
+{
+  GtdIdleData *idle_data = data;
+
+  g_clear_pointer (&idle_data->added, g_ptr_array_unref);
+  g_clear_pointer (&idle_data->removed, g_ptr_array_unref);
+  g_clear_pointer (&idle_data, g_free);
+}
+
+static void
+add_and_remove_tasks_in_idle (GtdTaskListView *self,
+                              GPtrArray       *added,
+                              GPtrArray       *removed)
+{
+  GtdTaskListViewPrivate *priv = gtd_task_list_view_get_instance_private (self);
+  GtdIdleData *idle_data;
+
+  /* If there's nothing to add or remove, don't do anything */
+  if (added->len == 0 && removed->len == 0)
+    return;
+
+  idle_data = g_new0 (GtdIdleData, 1);
+  idle_data->self = self;
+  idle_data->added = g_ptr_array_ref (added);
+  idle_data->removed = g_ptr_array_ref (removed);
+  idle_data->state = GTD_IDLE_STATE_STARTED;
+
+  /* Start loading stuff in idle */
+  priv->idle_handler_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                                           idle_process_items_cb,
+                                           idle_data,
+                                           idle_data_free);
+}
+
 static void
 set_active_row (GtdTaskListView *self,
                 GtkWidget       *row)
@@ -315,12 +375,13 @@ ask_subtask_removal_warning (GtdTaskListView *self)
 
 static void
 add_task_row (GtdTaskListView *self,
-              GtdTask         *task)
+              GtdTask         *task,
+              gboolean         animated)
 {
   GtdTaskListViewPrivate *priv = self->priv;
   GtkWidget *new_row;
 
-  new_row = gtd_task_row_new (task);
+  new_row = gtd_task_row_new (task, priv->renderer);
 
   g_object_bind_property (self,
                           "handle-subtasks",
@@ -356,52 +417,21 @@ add_task_row (GtdTaskListView *self,
                                priv->tasklist_name_sizegroup,
                                priv->due_date_sizegroup);
 
-  gtd_task_row_reveal (GTD_TASK_ROW (new_row));
+  gtd_task_row_reveal (GTD_TASK_ROW (new_row), animated);
 }
 
 static void
 update_state (GtdTaskListView *self)
 {
   GtdTaskListViewPrivate *priv;
-  g_autofree gchar *new_label = NULL;
-  GList *l;
-  gboolean is_empty;
-  gint complete_tasks;
 
   g_assert (GTD_IS_TASK_LIST_VIEW (self));
 
   priv = gtd_task_list_view_get_instance_private (self);
 
-  /* Update the completed tasks counter */
-  complete_tasks = 0;
-  is_empty = TRUE;
-
-  for (l = priv->list; l; l = l->next)
-    {
-      complete_tasks += gtd_task_get_complete (l->data);
-
-      if (priv->show_completed || !gtd_task_get_complete (l->data))
-        is_empty = FALSE;
-    }
-
-  if (complete_tasks == 0)
-    new_label = g_strdup_printf ("%s", _("Done"));
-  else
-    new_label = g_strdup_printf (_("Done (%d)"), complete_tasks);
-
-  /* Set the new counter */
-  gtk_label_set_label (priv->done_label, new_label);
-  gtk_revealer_set_reveal_child (GTK_REVEALER (priv->revealer), complete_tasks > 0);
-
   /* And the empty widgets */
-  gtk_widget_set_visible (priv->empty_box, is_empty);
-  gtd_empty_list_widget_set_is_empty (GTD_EMPTY_LIST_WIDGET (priv->empty_box), complete_tasks == 0);
-
-  /*
-   * If there are visible tasks, we visually separate the New Task row by
-   * increasing the top margin to 12px.
-   */
-  gtk_widget_set_margin_top (GTK_WIDGET (priv->new_task_row), is_empty ? 3 : 12);
+  gtk_widget_set_visible (priv->empty_box, FALSE);
+  gtd_empty_list_widget_set_is_empty (GTD_EMPTY_LIST_WIDGET (priv->empty_box), FALSE);
 }
 
 static GtdTaskRow*
@@ -441,37 +471,20 @@ destroy_task_row (GtdTaskListView *self,
 }
 
 static void
-remove_task (GtdTaskListView *view,
+remove_task (GtdTaskListView *self,
              GtdTask         *task)
 {
-  GtdTaskListViewPrivate *priv = view->priv;
+  GtdTaskListViewPrivate *priv = gtd_task_list_view_get_instance_private (self);
   GtdTaskRow *row;
 
-  row = get_row_for_task (view, task);
+  row = get_row_for_task (self, task);
+
+  /* We must not try to remove a task that is not added */
+  g_assert (g_hash_table_contains (priv->tasks, task));
+  g_hash_table_remove (priv->tasks, task);
 
   if (row)
-    {
-      g_signal_handlers_disconnect_by_func (task, on_task_completed_cb, view);
-      destroy_task_row (view, row);
-    }
-
-  gtk_revealer_set_reveal_child (priv->revealer, FALSE);
-}
-
-static inline gboolean
-has_complete_parent (GtdTask *task)
-{
-  GtdTask *parent = gtd_task_get_parent (task);
-
-  while (parent)
-    {
-      if (gtd_task_get_complete (parent))
-        return TRUE;
-
-      parent = gtd_task_get_parent (parent);
-    }
-
-  return FALSE;
+    destroy_task_row (self, row);
 }
 
 static void
@@ -480,8 +493,8 @@ remove_task_row (GtdTaskListView *view,
 {
   GtdTaskRow *row;
 
-  g_return_if_fail (GTD_IS_TASK_LIST_VIEW (view));
-  g_return_if_fail (GTD_IS_TASK (task));
+  g_assert (GTD_IS_TASK_LIST_VIEW (view));
+  g_assert (GTD_IS_TASK (task));
 
   row = get_row_for_task (view, task);
 
@@ -490,34 +503,163 @@ remove_task_row (GtdTaskListView *view,
 }
 
 static void
-add_task (GtdTaskListView *view,
-          GtdTask         *task)
+add_task (GtdTaskListView *self,
+          GtdTask         *task,
+          gboolean         animated)
 {
-  GtdTaskListViewPrivate *priv = view->priv;
+  GtdTaskListViewPrivate *priv = gtd_task_list_view_get_instance_private (self);
 
   GTD_ENTRY;
 
-  g_return_if_fail (GTD_IS_TASK_LIST_VIEW (view));
+  g_return_if_fail (GTD_IS_TASK_LIST_VIEW (self));
   g_return_if_fail (GTD_IS_TASK (task));
 
-  if (!priv->show_completed &&
-      (gtd_task_get_complete (task) || has_complete_parent (task)))
-    {
-      GTD_RETURN ();
-    }
+  /* We must not try to add a task that is already added */
+  g_assert (!g_hash_table_contains (priv->tasks, task));
+  g_hash_table_add (priv->tasks, task);
 
-  add_task_row (view, task);
-
-  update_state (view);
+  add_task_row (self, task, animated);
+  update_state (self);
 
   GTD_EXIT;
 }
 
+static void
+schedule_scroll_to_bottom (GtdTaskListView *self)
+{
+  GtdTaskListViewPrivate *priv = gtd_task_list_view_get_instance_private (self);
+
+  if (priv->scroll_to_bottom_handler_id > 0)
+    return;
+
+  priv->scroll_to_bottom_handler_id = g_timeout_add (250, scroll_to_bottom_cb, self);
+}
+
 
 /*
  * Callbacks
  */
 
+static gboolean
+scroll_to_bottom_cb (gpointer data)
+{
+  GtdTaskListViewPrivate *priv;
+  GtkWindow *window;
+  GtkWidget *widget;
+
+  priv = gtd_task_list_view_get_instance_private (data);
+  widget = GTK_WIDGET (data);
+  window = GTK_WINDOW (gtk_widget_get_toplevel (widget));
+
+  g_assert (window != NULL);
+
+  priv->scroll_to_bottom_handler_id = 0;
+
+  /*
+   * Only focus the new task row if the current list is visible,
+   * and the focused widget isn't inside this list view.
+   */
+  if (gtk_widget_get_visible (widget) &&
+      gtk_widget_get_child_visible (widget) &&
+      gtk_widget_get_mapped (widget) &&
+      !gtk_widget_is_ancestor (gtk_window_get_focus (window), widget))
+    {
+      gboolean ignored;
+
+      gtk_widget_grab_focus (GTK_WIDGET (priv->new_task_row));
+      g_signal_emit_by_name (priv->scrolled_window, "scroll-child", GTK_SCROLL_END, FALSE, &ignored);
+    }
+
+  return G_SOURCE_REMOVE;
+}
+
+static gboolean
+idle_process_items_cb (gpointer data)
+{
+  GtdTaskListViewPrivate *priv;
+  GtdIdleData *idle_data;
+  guint32 n_removed_items;
+  guint32 n_added_items;
+  guint i;
+
+  idle_data = data;
+  priv = gtd_task_list_view_get_instance_private (idle_data->self);
+
+  g_assert (idle_data->state == GTD_IDLE_STATE_STARTED || idle_data->state == GTD_IDLE_STATE_LOADING);
+
+  n_removed_items = idle_data->removed->len;
+  n_added_items = idle_data->added->len;
+
+  /* Stop if we finished loading */
+  if (idle_data->current_item == n_removed_items + n_added_items)
+    {
+      GTD_TRACE_MSG ("Finished adding %u and removing %u items", n_added_items, n_removed_items);
+
+      /* Check if it should show the empty state */
+      update_state (idle_data->self);
+
+      priv->idle_handler_id = 0;
+      idle_data->state = GTD_IDLE_STATE_FINISHED;
+
+      /* Show the list again */
+      gtk_widget_show (GTK_WIDGET (priv->listbox));
+      gtk_stack_set_visible_child_name (priv->stack, "listbox");
+
+      /* Scroll to the bottom */
+      schedule_scroll_to_bottom (idle_data->self);
+
+      return G_SOURCE_REMOVE;
+    }
+
+  GTD_TRACE_MSG ("Processing items (removed: %u, added: %u, current: %u)",
+                 n_removed_items,
+                 n_added_items,
+                 idle_data->current_item);
+
+  idle_data->state = GTD_IDLE_STATE_LOADING;
+
+  /*
+   * A performance trick we do is to hide the listbox until it's completely
+   * finished. Only do that if we're going to idle more than 30 tasks. This
+   * number is arbitrary
+   */
+  if (n_added_items >= N_TASKS_BEFORE_LOAD)
+    {
+      gtk_widget_hide (GTK_WIDGET (priv->listbox));
+      gtk_stack_set_visible_child_name (priv->stack, "loading");
+    }
+
+  /*
+   * Here we take the following approach:
+   *  1. Remove all the rows marked for removal at once.
+   *  2. Add the remaining rows in chunks of BATCH_SIZE
+   */
+  if (idle_data->current_item < n_removed_items)
+    {
+      for (i = 0; i < n_removed_items; i++)
+        remove_task (idle_data->self, g_ptr_array_index (idle_data->removed, i));
+
+      idle_data->current_item = n_removed_items;
+    }
+
+  /* No tasks to add, run the mainloop one more time to finish loading */
+  if (n_added_items == 0)
+    return G_SOURCE_CONTINUE;
+
+  /*
+   * For performance reasons, only animate task reveal when adding less
+   * than a batch of items.
+   */
+  add_task (idle_data->self,
+            g_ptr_array_index (idle_data->added, idle_data->current_item - n_removed_items),
+            n_added_items < N_TASKS_BEFORE_LOAD);
+
+  /* Next item */
+  idle_data->current_item += 1;
+
+  return G_SOURCE_CONTINUE;
+}
+
 static gboolean
 undo_remove_task_cb (GtdTaskListView *self,
                      GtdTask         *task)
@@ -539,7 +681,7 @@ real_remove_task_cb (GtdTaskListView *self,
   gtd_provider_remove_task (gtd_task_get_provider (task), task);
 
   /* Remove from the internal list */
-  priv->list = g_list_remove (priv->list, task);
+  g_hash_table_remove (priv->tasks, task);
 
   GTD_TRACE_MSG ("Removing task %p from list", task);
 
@@ -619,7 +761,7 @@ remove_task_rows_from_list_view_cb (GtdTaskListView *self,
   on_task_list_task_removed_cb (self, task);
 
   /* Remove from the internal list */
-  priv->list = g_list_remove (priv->list, task);
+  g_hash_table_remove (priv->tasks, task);
 
   GTD_TRACE_MSG ("Removing task %p from list", task);
 
@@ -687,6 +829,7 @@ on_remove_task_row_cb (GtdTaskRow      *row,
 
   gtd_window_notify (window, notification);
 
+
   /* Clear the active row */
   set_active_row (self, NULL);
 
@@ -744,7 +887,6 @@ on_done_button_clicked_cb (GtkButton *button,
                            gpointer   user_data)
 {
   GtdTaskListView *view = GTD_TASK_LIST_VIEW (user_data);
-  gboolean show_completed;
 
   g_return_if_fail (GTD_IS_TASK_LIST_VIEW (view));
 
@@ -759,10 +901,6 @@ on_done_button_clicked_cb (GtkButton *button,
    */
   view->priv->can_toggle = FALSE;
 
-  show_completed = view->priv->show_completed;
-
-  gtd_task_list_view_set_show_completed (view, !show_completed);
-
   g_timeout_add (205, (GSourceFunc) on_can_toggle_show_completed_cb, user_data);
 }
 
@@ -795,12 +933,10 @@ on_task_list_task_added_cb (GtdTaskList     *list,
 
   GTD_ENTRY;
 
-  add_task (self, task);
+  add_task (self, task, TRUE);
 
   /* Also add to the list of current tasks */
-  priv->list = g_list_prepend (priv->list, task);
-
-  g_signal_connect (task, "notify::complete", G_CALLBACK (on_task_completed_cb), self);
+  g_hash_table_add (priv->tasks, task);
 
   GTD_TRACE_MSG ("Adding task %p to list", task);
 
@@ -818,6 +954,30 @@ on_task_list_task_removed_cb (GtdTaskListView *view,
   update_state (view);
 }
 
+static void
+on_listbox_row_activated_cb (GtkListBox      *listbox,
+                             GtkListBox      *row,
+                             GtdTaskListView *self)
+{
+  if (!GTD_IS_TASK_ROW (row))
+    return;
+
+  /* Toggle the row */
+  if (gtd_task_row_get_active (GTD_TASK_ROW (row)))
+    set_active_row (self, NULL);
+  else
+    set_active_row (self, GTK_WIDGET (row));
+}
+
+static gint
+sort_tasks_gptrarray_cb (gconstpointer a,
+                         gconstpointer b)
+{
+  GtdTask *task_a = (*((GtdTask**) a));
+  GtdTask *task_b = (*((GtdTask**) b));
+
+  return gtd_task_compare (task_a, task_b);
+}
 
 /*
  * Default sorting functions
@@ -828,18 +988,13 @@ compare_task_rows (GtkListBoxRow *row1,
                    GtkListBoxRow *row2)
 {
   if (GTD_IS_NEW_TASK_ROW (row1))
-    {
-      return 1;
-    }
-  else if (GTD_IS_NEW_TASK_ROW (row2))
-    {
-      return -1;
-    }
-  else
-    {
-      return gtd_task_compare (gtd_task_row_get_task (GTD_TASK_ROW (row1)),
-                               gtd_task_row_get_task (GTD_TASK_ROW (row2)));
-    }
+    return 1;
+
+  if (GTD_IS_NEW_TASK_ROW (row2))
+    return -1;
+
+  return gtd_task_compare (gtd_task_row_get_task (GTD_TASK_ROW (row1)),
+                           gtd_task_row_get_task (GTD_TASK_ROW (row2)));
 }
 
 static gint
@@ -883,6 +1038,28 @@ default_sort_func (GtkListBoxRow *row1,
   return compare_task_rows (row1, row2);
 }
 
+
+/*
+ * Custom filter function
+ */
+
+static gboolean
+custom_filter_func (GtkListBoxRow   *row,
+                    GtdTaskListView *self)
+{
+  GtdTaskListViewPrivate *priv;
+  GtdTask *task;
+
+  if (GTD_IS_NEW_TASK_ROW (row) || GTD_IS_DND_ROW (row))
+    return gtk_widget_get_visible (GTK_WIDGET (row));
+
+  priv = gtd_task_list_view_get_instance_private (self);
+  task = gtd_task_row_get_task (GTD_TASK_ROW (row));
+
+  return priv->filter_func (task, priv->filter_user_data);;
+}
+
+
 /*
  * Custom sorting functions
  */
@@ -985,63 +1162,6 @@ custom_sort_func (GtkListBoxRow   *a,
   return internal_compare_task_rows (view, a, b);
 }
 
-static void
-listbox_row_activated (GtkListBox      *listbox,
-                       GtkListBox      *row,
-                       GtdTaskListView *self)
-{
-  if (!GTD_IS_TASK_ROW (row))
-    return;
-
-  /* Toggle the row */
-  if (gtd_task_row_get_active (GTD_TASK_ROW (row)))
-    set_active_row (self, NULL);
-  else
-    set_active_row (self, GTK_WIDGET (row));
-}
-
-static inline gboolean
-remove_subtasks_of_completed_task_cb (GtdTaskListView *self,
-                                      GtdTask         *task)
-{
-  remove_task_row (self, task);
-  return TRUE;
-}
-
-static inline gboolean
-add_subtasks_of_task_cb (GtdTaskListView *self,
-                         GtdTask         *task)
-{
-  add_task (self, task);
-  return TRUE;
-}
-
-static void
-on_task_completed_cb (GtdTask         *task,
-                      GParamSpec      *spec,
-                      GtdTaskListView *self)
-{
-  GtdTaskListViewPrivate *priv = gtd_task_list_view_get_instance_private (self);
-  gboolean completed;
-
-  completed = gtd_task_get_complete (task);
-
-  if (!priv->show_completed)
-    {
-      IterateSubtaskFunc func;
-
-      /* Task already has a row, no need to iterate */
-      if (!completed && get_row_for_task (self, task))
-        return;
-
-      func = completed ? remove_subtasks_of_completed_task_cb : add_subtasks_of_task_cb;
-
-      iterate_subtasks (self, task, func);
-    }
-
-  update_state (self);
-}
-
 
 /*
  * Drag n' Drop functions
@@ -1296,8 +1416,21 @@ gtd_task_list_view_finalize (GObject *object)
 {
   GtdTaskListViewPrivate *priv = GTD_TASK_LIST_VIEW (object)->priv;
 
+  if (priv->idle_handler_id > 0)
+    {
+      g_source_remove (priv->idle_handler_id);
+      priv->idle_handler_id = 0;
+    }
+
+  if (priv->scroll_to_bottom_handler_id > 0)
+    {
+      g_source_remove (priv->scroll_to_bottom_handler_id);
+      priv->scroll_to_bottom_handler_id = 0;
+    }
+
   g_clear_pointer (&priv->default_date, g_date_time_unref);
-  g_clear_pointer (&priv->list, g_list_free);
+  g_clear_pointer (&priv->tasks, g_hash_table_destroy);
+  g_clear_object (&priv->renderer);
 
   G_OBJECT_CLASS (gtd_task_list_view_parent_class)->finalize (object);
 }
@@ -1320,10 +1453,6 @@ gtd_task_list_view_get_property (GObject    *object,
       g_value_set_boolean (value, self->priv->handle_subtasks);
       break;
 
-    case PROP_SHOW_COMPLETED:
-      g_value_set_boolean (value, self->priv->show_completed);
-      break;
-
     case PROP_SHOW_DUE_DATE:
       g_value_set_boolean (value, self->priv->show_due_date);
       break;
@@ -1359,10 +1488,6 @@ gtd_task_list_view_set_property (GObject      *object,
       gtd_task_list_view_set_handle_subtasks (self, g_value_get_boolean (value));
       break;
 
-    case PROP_SHOW_COMPLETED:
-      gtd_task_list_view_set_show_completed (self, g_value_get_boolean (value));
-      break;
-
     case PROP_SHOW_DUE_DATE:
       gtd_task_list_view_set_show_due_date (self, g_value_get_boolean (value));
       break;
@@ -1512,20 +1637,6 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
                               FALSE,
                               G_PARAM_READWRITE));
 
-  /**
-   * GtdTaskListView::show-completed:
-   *
-   * Whether completed tasks are shown.
-   */
-  g_object_class_install_property (
-        object_class,
-        PROP_SHOW_COMPLETED,
-        g_param_spec_boolean ("show-completed",
-                              "Whether completed tasks are shown",
-                              "Whether completed tasks are visible or not",
-                              FALSE,
-                              G_PARAM_READWRITE));
-
   /**
    * GtdTaskListView::show-due-date:
    *
@@ -1546,18 +1657,16 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, due_date_sizegroup);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, empty_box);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, listbox);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, revealer);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, done_image);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, done_label);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, new_task_row);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, tasklist_name_sizegroup);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, viewport);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, scrolled_window);
+  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, stack);
+  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, viewport);
 
   gtk_widget_class_bind_template_callback (widget_class, listbox_drag_drop);
   gtk_widget_class_bind_template_callback (widget_class, listbox_drag_leave);
   gtk_widget_class_bind_template_callback (widget_class, listbox_drag_motion);
-  gtk_widget_class_bind_template_callback (widget_class, listbox_row_activated);
+  gtk_widget_class_bind_template_callback (widget_class, on_listbox_row_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_done_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_task_row_entered_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_task_row_exited_cb);
@@ -1568,10 +1677,16 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
 static void
 gtd_task_list_view_init (GtdTaskListView *self)
 {
-  self->priv = gtd_task_list_view_get_instance_private (self);
-  self->priv->can_toggle = TRUE;
-  self->priv->handle_subtasks = TRUE;
-  self->priv->show_due_date = TRUE;
+  GtdTaskListViewPrivate *priv = gtd_task_list_view_get_instance_private (self);
+
+  self->priv = priv;
+
+  priv->can_toggle = TRUE;
+  priv->handle_subtasks = TRUE;
+  priv->show_due_date = TRUE;
+  priv->show_due_date = TRUE;
+
+  priv->tasks = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -1582,6 +1697,8 @@ gtd_task_list_view_init (GtdTaskListView *self)
                      NULL,
                      0,
                      GDK_ACTION_MOVE);
+
+  self->priv->renderer = gtd_markdown_renderer_new ();
 }
 
 /**
@@ -1615,10 +1732,8 @@ gtd_task_list_view_get_list (GtdTaskListView *view)
 
   if (view->priv->task_list)
     return gtd_task_list_get_tasks (view->priv->task_list);
-  else if (view->priv->list)
-    return g_list_copy (view->priv->list);
   else
-    return NULL;
+    return g_hash_table_get_keys (view->priv->tasks);
 }
 
 /**
@@ -1632,40 +1747,65 @@ void
 gtd_task_list_view_set_list (GtdTaskListView *view,
                              GList           *list)
 {
+  g_autoptr (GHashTable) new_tasks = NULL;
+  g_autoptr (GPtrArray) removed = NULL;
+  g_autoptr (GPtrArray) added = NULL;
+  g_autoptr (GList) old_list = NULL;
   GtdTaskListViewPrivate *priv;
-  GList *l, *old_list;
+  GList *l;
 
   g_return_if_fail (GTD_IS_TASK_LIST_VIEW (view));
 
-  priv = view->priv;
-  old_list = priv->list;
+  priv = gtd_task_list_view_get_instance_private (view);
+
+  /*
+   * If we are currently adding tasks in idle, cancel the idle before
+   * anything else. Just for safety.
+   */
+  if (priv->idle_handler_id > 0)
+    {
+      GTD_TRACE_MSG ("Cancelling current idle add");
+
+      g_source_remove (priv->idle_handler_id);
+      priv->idle_handler_id = 0;
+    }
+
+  added = g_ptr_array_new_full (100, g_object_unref);
+  removed = g_ptr_array_new_full (100, g_object_unref);
+  new_tasks = g_hash_table_new (g_direct_hash, g_direct_equal);
+  old_list = g_hash_table_get_keys (priv->tasks);
 
   /* Reset the DnD parent row */
   gtd_dnd_row_set_row_above (GTD_DND_ROW (priv->dnd_row), NULL);
 
+  /* Map the current tasks */
+  for (l = list; l; l = l->next)
+    g_hash_table_add (new_tasks, l->data);
+
   /* Remove the tasks that are in the current list, but not in the new list */
-  for (l = old_list; l != NULL; l = l->next)
+  for (l = old_list; l; l = l->next)
     {
-      if (!g_list_find (list, l->data))
-        remove_task (view, l->data);
+      if (!g_hash_table_contains (new_tasks, l->data))
+        g_ptr_array_add (removed, g_object_ref (l->data));
     }
 
   /* Add the tasks that are in the new list, but not in the current list */
-  for (l = list; l != NULL; l = l->next)
+  for (l = list; l; l = l->next)
     {
-      if (g_list_find (old_list, l->data))
+      if (g_hash_table_contains (priv->tasks, l->data))
         continue;
 
-      add_task (view, l->data);
-
-      g_signal_connect (l->data, "notify::complete", G_CALLBACK (on_task_completed_cb), view);
+      g_ptr_array_add (added, g_object_ref (l->data));
     }
 
-  g_list_free (old_list);
-  priv->list = g_list_copy (list);
+  /* Sort the array so we can add the tasks in the order they'll be displayed */
+  g_ptr_array_sort (added, sort_tasks_gptrarray_cb);
 
-  /* Check if it should show the empty state */
-  update_state (view);
+  /*
+   * Add the new tasks and remove the old ones in idle, to keep
+   * a slick UI responsiveness.
+   */
+  add_and_remove_tasks_in_idle (view, added, removed);
 }
 
 /**
@@ -1728,17 +1868,21 @@ void
 gtd_task_list_view_set_task_list (GtdTaskListView *view,
                                   GtdTaskList     *list)
 {
-  GtdTaskListViewPrivate *priv = view->priv;
-  GdkRGBA *color;
-  gchar *color_str;
-  gchar *parsed_css;
-  GList *task_list;
+  GtdTaskListViewPrivate *priv;
+  g_autoptr (GdkRGBA) color = NULL;
+  g_autoptr (GList) task_list = NULL;
+  g_autofree gchar *parsed_css = NULL;
+  g_autofree gchar *color_str = NULL;
 
   g_return_if_fail (GTD_IS_TASK_LIST_VIEW (view));
 
+  priv = gtd_task_list_view_get_instance_private (view);
+
   if (priv->task_list == list)
     return;
 
+  g_debug ("%p: Setting task list to '%s'", view, gtd_task_list_get_name (list));
+
   gtd_new_task_row_set_show_list_selector (GTD_NEW_TASK_ROW (priv->new_task_row), list == NULL);
 
   /*
@@ -1768,19 +1912,12 @@ gtd_task_list_view_set_task_list (GtdTaskListView *view,
                                    -1,
                                    NULL);
 
-  g_free (parsed_css);
-  gdk_rgba_free (color);
-  g_free (color_str);
-
   update_font_color (view);
 
   /* Add the tasks from the list */
   task_list = gtd_task_list_get_tasks (list);
-
   gtd_task_list_view_set_list (view, task_list);
 
-  g_list_free (task_list);
-
   g_signal_connect (list,
                     "task-added",
                     G_CALLBACK (on_task_list_task_added_cb),
@@ -1910,100 +2047,6 @@ gtd_task_list_view_set_show_due_date (GtdTaskListView *self,
   g_object_notify (G_OBJECT (self), "show-due-date");
 }
 
-/**
- * gtd_task_list_view_get_show_completed:
- * @view: a #GtdTaskListView
- *
- * Returns %TRUE if completed tasks are visible, %FALSE otherwise.
- *
- * Returns: %TRUE if completed tasks are visible, %FALSE if they are hidden
- */
-gboolean
-gtd_task_list_view_get_show_completed (GtdTaskListView *view)
-{
-  g_return_val_if_fail (GTD_IS_TASK_LIST_VIEW (view), FALSE);
-
-  return view->priv->show_completed;
-}
-
-/**
- * gtd_task_list_view_set_show_completed:
- * @view: a #GtdTaskListView
- * @show_completed: %TRUE to show completed tasks, %FALSE to hide them
- *
- * Sets the ::show-completed property to @show_completed.
- */
-void
-gtd_task_list_view_set_show_completed (GtdTaskListView *view,
-                                       gboolean         show_completed)
-{
-  GtdTaskListViewPrivate *priv = view->priv;
-
-  g_return_if_fail (GTD_IS_TASK_LIST_VIEW (view));
-
-  if (priv->show_completed != show_completed)
-    {
-
-      priv->show_completed = show_completed;
-
-      gtk_image_set_from_icon_name (view->priv->done_image,
-                                    show_completed ? "zoom-out-symbolic" : "zoom-in-symbolic",
-                                    GTK_ICON_SIZE_BUTTON);
-
-
-      /* insert or remove list rows */
-      if (show_completed)
-        {
-          GList *list_of_tasks;
-          GList *l;
-
-          list_of_tasks = gtd_task_list_view_get_list (view);
-
-          for (l = list_of_tasks; l != NULL; l = l->next)
-            {
-              /*
-               * Consider that not-complete tasks, and non-complete tasks with a non-complete
-               * parent, are already present.
-               */
-              if (!gtd_task_get_complete (l->data) && !has_complete_parent (l->data))
-                continue;
-
-              add_task_row (view, l->data);
-            }
-
-            g_list_free (list_of_tasks);
-        }
-      else
-        {
-          GList *children;
-          GList *l;
-
-          children = gtk_container_get_children (GTK_CONTAINER (priv->listbox));
-
-          for (l = children; l != NULL; l = l->next)
-            {
-              GtdTask *task;
-
-              if (!GTD_IS_TASK_ROW (l->data))
-                continue;
-
-              task = gtd_task_row_get_task (l->data);
-
-              /* Remove completed tasks, and also tasks with a completed parent */
-              if (gtd_task_get_complete (task) || has_complete_parent (task))
-                destroy_task_row (view, l->data);
-            }
-
-          g_list_free (children);
-        }
-
-      /* Check if it should show the empty state */
-      update_state (view);
-
-      g_object_notify (G_OBJECT (view), "show-completed");
-    }
-}
-
 /**
  * gtd_task_list_view_set_header_func:
  * @view: a #GtdTaskListView
@@ -2048,6 +2091,46 @@ gtd_task_list_view_set_header_func (GtdTaskListView           *view,
     }
 }
 
+/**
+ * gtd_task_list_view_set_filter_func:
+ * @view: a #GtdTaskListView
+ * @func: (closure user_data) (scope call) (nullable): the filter function
+ * @user_data: data passed to @func
+ *
+ * Sets @func as the filter function of @view.
+ *
+ * Do not unref nor free any of the passed data.
+ */
+void
+gtd_task_list_view_set_filter_func (GtdTaskListView           *view,
+                                    GtdTaskListViewFilterFunc  func,
+                                    gpointer                   user_data)
+{
+  GtdTaskListViewPrivate *priv;
+
+  g_return_if_fail (GTD_IS_TASK_LIST_VIEW (view));
+
+  priv = view->priv;
+
+  if (func)
+    {
+      priv->filter_func = func;
+      priv->filter_user_data = user_data;
+
+      gtk_list_box_set_filter_func (priv->listbox,
+                                    (GtkListBoxFilterFunc) custom_filter_func,
+                                    view,
+                                    NULL);
+    }
+  else
+    {
+      priv->filter_func = NULL;
+      priv->filter_user_data = NULL;
+
+      gtk_list_box_set_filter_func (priv->listbox, NULL, NULL, NULL);
+    }
+}
+
 /**
  * gtd_task_list_view_set_sort_func:
  * @view: a #GtdTaskListView
@@ -2254,4 +2337,5 @@ gtd_task_list_view_invalidate (GtdTaskListView *self)
 
   gtk_list_box_invalidate_sort (priv->listbox);
   gtk_list_box_invalidate_headers (priv->listbox);
+  gtk_list_box_invalidate_filter (priv->listbox);
 }
