@@ -22,6 +22,7 @@
 #include "gtd-edit-pane.h"
 #include "gtd-expandable-entry.h"
 #include "gtd-manager.h"
+#include "gtd-markdown-renderer.h"
 #include "gtd-provider.h"
 #include "gtd-rows-common-private.h"
 #include "gtd-task-row.h"
@@ -40,7 +41,6 @@ struct _GtdTaskRow
   GtkRevealer        *revealer;
 
   GtkWidget          *done_check;
-  GtkWidget          *edit_panel;
   GtkWidget          *edit_panel_revealer;
   GtkWidget          *header_event_box;
   GtkWidget          *title_entry;
@@ -62,6 +62,10 @@ struct _GtdTaskRow
   /* data */
   GtdTask            *task;
 
+  GtdEditPane        *edit_pane;
+
+  GtdMarkdownRenderer *renderer;
+
   gboolean            active;
   gboolean            changed;
 };
@@ -103,6 +107,7 @@ enum
 {
   PROP_0,
   PROP_HANDLE_SUBTASKS,
+  PROP_RENDERER,
   PROP_TASK,
   LAST_PROP
 };
@@ -176,7 +181,7 @@ create_transient_row (GtdTaskRow *self)
 {
   GtdTaskRow *new_row;
 
-  new_row = GTD_TASK_ROW (gtd_task_row_new (self->task));
+  new_row = GTD_TASK_ROW (gtd_task_row_new (self->task, self->renderer));
   gtk_revealer_set_transition_duration (new_row->revealer, 0);
   gtk_revealer_set_reveal_child (new_row->revealer, TRUE);
 
@@ -254,27 +259,28 @@ gtd_task_row_set_task (GtdTaskRow *row,
        * sync the initial state of the priority icon.
        */
       on_priority_changed_cb (row, NULL, G_OBJECT (task));
-      g_signal_connect_swapped (task,
-                                "notify::priority",
-                                G_CALLBACK (on_priority_changed_cb),
-                                row);
+      g_signal_connect_object (task,
+                               "notify::priority",
+                               G_CALLBACK (on_priority_changed_cb),
+                               row,
+                               G_CONNECT_SWAPPED);
 
       on_complete_changed_cb (row, NULL, task);
-      g_signal_connect_swapped (task,
-                                "notify::complete",
-                                G_CALLBACK (on_complete_changed_cb),
-                                row);
+      g_signal_connect_object (task,
+                               "notify::complete",
+                               G_CALLBACK (on_complete_changed_cb),
+                               row,
+                               G_CONNECT_SWAPPED);
 
       on_depth_changed_cb (row, NULL, task);
-      g_signal_connect_swapped (task,
-                                "notify::depth",
-                                G_CALLBACK (on_depth_changed_cb),
-                                row);
+      g_signal_connect_object (task,
+                               "notify::depth",
+                               G_CALLBACK (on_depth_changed_cb),
+                               row,
+                               G_CONNECT_SWAPPED);
 
       g_signal_handlers_unblock_by_func (row->done_check, on_complete_check_toggled_cb, row);
       g_signal_handlers_unblock_by_func (row->title_entry, on_task_changed_cb, row);
-
-      gtd_edit_pane_set_task (GTD_EDIT_PANE (row->edit_panel), task);
     }
 
   g_object_notify (G_OBJECT (row), "task");
@@ -509,7 +515,6 @@ on_task_changed_cb (GtdTaskRow  *self)
   self->changed = TRUE;
 }
 
-
 /*
  * GtkWidget overrides
  */
@@ -585,6 +590,10 @@ gtd_task_row_get_property (GObject    *object,
       g_value_set_boolean (value, self->handle_subtasks);
       break;
 
+    case PROP_RENDERER:
+      g_value_set_object (value, self->renderer);
+      break;
+
     case PROP_TASK:
       g_value_set_object (value, self->task);
       break;
@@ -608,6 +617,10 @@ gtd_task_row_set_property (GObject      *object,
       gtd_task_row_set_handle_subtasks (self, g_value_get_boolean (value));
       break;
 
+    case PROP_RENDERER:
+      self->renderer = g_value_get_object (value);
+      break;
+
     case PROP_TASK:
       gtd_task_row_set_task (self, g_value_get_object (value));
       break;
@@ -647,6 +660,20 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
                                 TRUE,
                                 G_PARAM_READWRITE));
 
+  /**
+   * GtdTaskRow::renderer:
+   *
+   * The internal markdown renderer.
+   */
+  g_object_class_install_property (
+          object_class,
+          PROP_RENDERER,
+          g_param_spec_object ("renderer",
+                               "Renderer",
+                               "Renderer",
+                               GTD_TYPE_MARKDOWN_RENDERER,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_PRIVATE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GtdTaskRow::task:
    *
@@ -712,7 +739,6 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, dnd_event_box);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, dnd_icon);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, done_check);
-  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, edit_panel);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, edit_panel_revealer);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, header_event_box);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, revealer);
@@ -761,10 +787,12 @@ gtd_task_row_init (GtdTaskRow *self)
 }
 
 GtkWidget*
-gtd_task_row_new (GtdTask *task)
+gtd_task_row_new (GtdTask             *task,
+                  GtdMarkdownRenderer *renderer)
 {
   return g_object_new (GTD_TYPE_TASK_ROW,
                        "task", task,
+                       "renderer", renderer,
                        NULL);
 }
 
@@ -824,10 +852,14 @@ gtd_task_row_set_due_date_visible (GtdTaskRow *row,
  * Runs a nifty animation to reveal @row.
  */
 void
-gtd_task_row_reveal (GtdTaskRow *row)
+gtd_task_row_reveal (GtdTaskRow *row,
+                     gboolean    animated)
 {
   g_return_if_fail (GTD_IS_TASK_ROW (row));
 
+  if (!animated)
+    gtk_revealer_set_transition_duration (row->revealer, 0);
+
   gtk_revealer_set_reveal_child (row->revealer, TRUE);
 }
 
@@ -842,8 +874,7 @@ gtd_task_row_destroy (GtdTaskRow *self)
 {
   g_return_if_fail (GTD_IS_TASK_ROW (self));
 
-  gtk_revealer_set_reveal_child (self->revealer, FALSE);
-  g_signal_connect_swapped (self->revealer, "notify::child-revealed", G_CALLBACK (gtk_widget_destroy), self);
+  gtk_widget_destroy (GTK_WIDGET (self));
 }
 
 gboolean
@@ -914,7 +945,30 @@ gtd_task_row_set_active (GtdTaskRow *self,
   else
     gtk_style_context_remove_class (context, "active");
 
-  /* And the listbox */
+  /* Create or destroy the edit panel */
+  if (active && !self->edit_pane)
+    {
+      GTD_TRACE_MSG ("Creating edit pane");
+
+      self->edit_pane = GTD_EDIT_PANE (gtd_edit_pane_new ());
+      gtd_edit_pane_set_markdown_renderer (self->edit_pane, self->renderer);
+      gtd_edit_pane_set_task (self->edit_pane, self->task);
+
+      gtk_container_add (GTK_CONTAINER (self->edit_panel_revealer), GTK_WIDGET (self->edit_pane));
+      gtk_widget_show (GTK_WIDGET (self->edit_pane));
+
+      g_signal_connect_swapped (self->edit_pane, "changed", G_CALLBACK (on_task_changed_cb), self);
+      g_signal_connect (self->edit_pane, "remove-task", G_CALLBACK (on_remove_task_cb), self);
+    }
+  else if (!active && self->edit_pane)
+    {
+      GTD_TRACE_MSG ("Destroying edit pane");
+
+      gtk_container_remove (GTK_CONTAINER (self->edit_panel_revealer), GTK_WIDGET (self->edit_pane));
+      self->edit_pane = NULL;
+    }
+
+  /* And reveal or hide it */
   gtk_revealer_set_reveal_child (GTK_REVEALER (self->edit_panel_revealer), active);
 
   /* Save the task if it is not being loaded */
