@@ -18,10 +18,11 @@
 
 #define G_LOG_DOMAIN "GtdWindow"
 
+#include "config.h"
+
 #include "interfaces/gtd-activatable.h"
 #include "interfaces/gtd-provider.h"
 #include "interfaces/gtd-panel.h"
-#include "views/gtd-list-selector-panel.h"
 #include "gtd-application.h"
 #include "gtd-enum-types.h"
 #include "gtd-task-list-view.h"
@@ -30,9 +31,12 @@
 #include "gtd-notification.h"
 #include "gtd-notification-widget.h"
 #include "gtd-plugin-manager.h"
+#include "gtd-sidebar.h"
 #include "gtd-task.h"
 #include "gtd-task-list.h"
+#include "gtd-task-list-panel.h"
 #include "gtd-window.h"
+#include "gtd-window-private.h"
 
 #include <glib/gi18n.h>
 
@@ -57,7 +61,7 @@ struct _GtdWindow
   GtkWidget          *gear_menu_button;
   GtkHeaderBar       *headerbar;
   GtkStack           *stack;
-  GtkStackSwitcher   *stack_switcher;
+  GtdSidebar         *sidebar;
 
   GtdNotificationWidget *notification_widget;
 
@@ -68,13 +72,11 @@ struct _GtdWindow
   GtkWidget          *panel_box_start;
 
   GtdPanel           *active_panel;
+  GtdPanel           *task_list_panel;
 
   /* mode */
   GtdWindowMode       mode;
 
-  /* loading notification */
-  GtdNotification    *loading_notification;
-
   guint               save_geometry_timeout_id;
 };
 
@@ -100,6 +102,28 @@ enum
 };
 
 
+static void
+setup_development_build (GtdWindow *self)
+{
+  GtkStyleContext *context;
+
+  g_message (_("This is a development build of To Do. You may experience errors, wrong behaviors, "
+               "and data loss."));
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (self));
+
+  gtk_style_context_add_class (context, "development-version");
+}
+
+static gboolean
+is_development_build (void)
+{
+#ifdef DEVELOPMENT_BUILD
+  return TRUE;
+#else
+  return FALSE;
+#endif
+}
 
 static void
 add_widgets (GtdWindow *self,
@@ -203,17 +227,6 @@ gtd_window__panel_menu_changed (GObject    *object,
   update_panel_menu (self);
 }
 
-static void
-gtd_window__panel_title_changed (GObject    *object,
-                                 GParamSpec *pspec,
-                                 GtdWindow  *self)
-{
-  gtk_container_child_set (GTK_CONTAINER (self->stack),
-                           GTK_WIDGET (object),
-                           "title", gtd_panel_get_panel_title (GTD_PANEL (object)),
-                           NULL);
-}
-
 static void
 on_panel_added_cb (GtdManager *manager,
                    GtdPanel   *panel,
@@ -223,11 +236,6 @@ on_panel_added_cb (GtdManager *manager,
                         GTK_WIDGET (panel),
                         gtd_panel_get_panel_name (panel),
                         gtd_panel_get_panel_title (panel));
-
-  g_signal_connect (panel,
-                    "notify::title",
-                    G_CALLBACK (gtd_window__panel_title_changed),
-                    self);
 }
 
 static void
@@ -435,19 +443,6 @@ on_stack_visible_child_cb (GtdWindow  *self,
   update_panel_menu (self);
 }
 
-static void
-on_loading_changed_cb (GObject    *source,
-                       GParamSpec *spec,
-                       GtdWindow  *self)
-{
-  g_assert (GTD_IS_WINDOW (self));
-
-  if (gtd_object_get_loading (GTD_OBJECT (source)))
-    gtd_notification_widget_notify (self->notification_widget, self->loading_notification);
-  else
-    gtd_notification_widget_cancel (self->notification_widget, self->loading_notification);
-}
-
 static void
 on_show_error_message_cb (GtdManager                *manager,
                           const gchar               *primary_text,
@@ -486,6 +481,14 @@ on_show_error_message_cb (GtdManager                *manager,
   gtd_window_notify (self, notification);
 }
 
+static void
+on_show_notification_cb (GtdManager      *manager,
+                         GtdNotification *notification,
+                         GtdWindow       *self)
+{
+  gtd_window_notify (self, notification);
+}
+
 
 /*
  * GtkWindow overrides
@@ -579,19 +582,11 @@ gtd_window_constructed (GObject *object)
   for (l = lists; l; l = l->next)
     on_panel_added_cb (NULL, l->data, self);
 
-  g_signal_connect (gtd_manager_get_default (), "notify::loading", G_CALLBACK (on_loading_changed_cb), self);
   g_signal_connect (gtd_manager_get_default (), "panel-added", G_CALLBACK (on_panel_added_cb), self);
   g_signal_connect (gtd_manager_get_default (), "panel-removed", G_CALLBACK (on_panel_removed_cb), self);
+
   g_signal_connect (gtd_manager_get_default (), "show-error-message", G_CALLBACK (on_show_error_message_cb), self);
-}
-
-static void
-gtd_window_finalize (GObject *object)
-{
-  GtdWindow *self = GTD_WINDOW (object);
-
-  g_clear_object (&self->loading_notification);
-  G_OBJECT_CLASS (gtd_window_parent_class)->finalize (object);
+  g_signal_connect (gtd_manager_get_default (), "show-notification", G_CALLBACK (on_show_notification_cb), self);
 }
 
 static void
@@ -638,7 +633,6 @@ gtd_window_class_init (GtdWindowClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-  object_class->finalize = gtd_window_finalize;
   object_class->constructed = gtd_window_constructed;
   object_class->get_property = gtd_window_get_property;
   object_class->set_property = gtd_window_set_property;
@@ -662,6 +656,7 @@ gtd_window_class_init (GtdWindowClass *klass)
                            G_PARAM_READWRITE));
 
   g_type_ensure (GTD_TYPE_NOTIFICATION_WIDGET);
+  g_type_ensure (GTD_TYPE_SIDEBAR);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/todo/ui/window.ui");
 
@@ -669,8 +664,8 @@ gtd_window_class_init (GtdWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GtdWindow, gear_menu_button);
   gtk_widget_class_bind_template_child (widget_class, GtdWindow, headerbar);
   gtk_widget_class_bind_template_child (widget_class, GtdWindow, notification_widget);
+  gtk_widget_class_bind_template_child (widget_class, GtdWindow, sidebar);
   gtk_widget_class_bind_template_child (widget_class, GtdWindow, stack);
-  gtk_widget_class_bind_template_child (widget_class, GtdWindow, stack_switcher);
 
   gtk_widget_class_bind_template_child (widget_class, GtdWindow, extension_box_end);
   gtk_widget_class_bind_template_child (widget_class, GtdWindow, extension_box_start);
@@ -684,23 +679,18 @@ gtd_window_class_init (GtdWindowClass *klass)
 static void
 gtd_window_init (GtdWindow *self)
 {
-  GtkWidget *panel;
-
-  self->loading_notification = gtd_notification_new (_("Loading your task lists…"), 0);
-  gtd_object_push_loading (GTD_OBJECT (self->loading_notification));
-
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  /* Add the default 'Lists' panel before everything else */
-  panel = gtd_list_selector_panel_new ();
+  /* Task list panel */
+  self->task_list_panel = GTD_PANEL (gtd_task_list_panel_new ());
+  on_panel_added_cb (gtd_manager_get_default (), self->task_list_panel, self);
 
-  on_panel_added_cb (gtd_manager_get_default (), GTD_PANEL (panel), self);
+  gtd_sidebar_set_panel_stack (self->sidebar, GTK_STACK (self->stack));
+  gtd_sidebar_set_task_list_panel (self->sidebar, self->task_list_panel);
 
-  g_object_bind_property (self,
-                          "mode",
-                          panel,
-                          "mode",
-                          G_BINDING_BIDIRECTIONAL);
+  /* Development build */
+  if (is_development_build ())
+    setup_development_build (self);
 }
 
 GtkWidget*
@@ -795,7 +785,6 @@ gtd_window_set_mode (GtdWindow     *self,
       else
         {
           gtk_style_context_remove_class (context, "selection-mode");
-          gtk_header_bar_set_custom_title (self->headerbar, GTK_WIDGET (self->stack_switcher));
           gtk_header_bar_set_title (self->headerbar, _("To Do"));
         }
 
@@ -828,6 +817,14 @@ gtd_window_set_custom_title (GtdWindow   *self,
   else
     {
       gtk_header_bar_set_title (self->headerbar, _("To Do"));
-      gtk_header_bar_set_custom_title (self->headerbar, GTK_WIDGET (self->stack_switcher));
     }
 }
+
+/* Private functions */
+void
+_gtd_window_finish_startup (GtdWindow *self)
+{
+  g_return_if_fail (GTD_IS_WINDOW (self));
+
+  gtd_sidebar_activate (self->sidebar);
+}
