@@ -37,6 +37,11 @@
  * in the headerbar according to the #GtkWidget:halign property. See
  * gtd_panel_get_header_widgets() for a detailed explanation.
  *
+ * The #GtdPanel:icon and #GtdPanel:priority properties are used by
+ * the sidebar. The former is used to display the icon, and the latter
+ * is used to determine the position of the panel relative to the
+ * others panels.
+ *
  * At last, a #GtdPanel implementation may provide a #GMenu that will
  * be appended to the window menu.
  */
@@ -46,6 +51,19 @@ G_DEFINE_INTERFACE (GtdPanel, gtd_panel, GTK_TYPE_WIDGET)
 static void
 gtd_panel_default_init (GtdPanelInterface *iface)
 {
+
+  /**
+   * GtdPanel::icon:
+   *
+   * The icon of the panel.
+   */
+  g_object_interface_install_property (iface,
+                                       g_param_spec_object ("icon",
+                                                            "Icon of the panel",
+                                                            "The icon of the panel",
+                                                            G_TYPE_ICON,
+                                                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GtdPanel::name:
    *
@@ -60,6 +78,33 @@ gtd_panel_default_init (GtdPanelInterface *iface)
                                                             NULL,
                                                             G_PARAM_READABLE));
 
+  /**
+   * GtdPanel::priority:
+   *
+   * The priority of the panel.
+   */
+  g_object_interface_install_property (iface,
+                                       g_param_spec_uint ("priority",
+                                                          "Priority of the panel",
+                                                          "The priority of the panel",
+                                                          0,
+                                                          G_MAXUINT32,
+                                                          0,
+                                                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GtdPanel::subtitle:
+   *
+   * The subtitle of the panel. It usually is the counter of not
+   * completed tasks.
+   */
+  g_object_interface_install_property (iface,
+                                       g_param_spec_string ("subtitle",
+                                                            "The subtitle of the panel",
+                                                            "The user-visible subtitle of the panel",
+                                                            NULL,
+                                                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GtdPanel::title:
    *
@@ -160,3 +205,55 @@ gtd_panel_get_menu (GtdPanel *panel)
 
   return GTD_PANEL_GET_IFACE (panel)->get_menu (panel);
 }
+
+/**
+ * gtd_panel_get_icon:
+ * @self: a #GtdPanel
+ *
+ * Retrieves the icon of @self.
+ *
+ * Returns: (transfer full)(nullable): a #GIcon
+ */
+GIcon*
+gtd_panel_get_icon (GtdPanel *self)
+{
+  g_return_val_if_fail (GTD_IS_PANEL (self), NULL);
+  g_return_val_if_fail (GTD_PANEL_GET_IFACE (self)->get_icon, NULL);
+
+  return GTD_PANEL_GET_IFACE (self)->get_icon (self);
+}
+
+/**
+ * gtd_panel_get_priority:
+ * @self: a #GtdPanel
+ *
+ * Retrieves the priority of @self. This value is used to
+ * determine the position of the panel in the sidebar.
+ *
+ * Returns: the priority of @self
+ */
+guint32
+gtd_panel_get_priority (GtdPanel *self)
+{
+  g_return_val_if_fail (GTD_IS_PANEL (self), 0);
+  g_return_val_if_fail (GTD_PANEL_GET_IFACE (self)->get_priority, 0);
+
+  return GTD_PANEL_GET_IFACE (self)->get_priority (self);
+}
+
+/**
+ * gtd_panel_get_subtitle:
+ * @panel: a #GtdPanel
+ *
+ * Retrieves the subtitle of @panel
+ *
+ * Returns: (transfer full): the subtitle of @panel
+ */
+gchar*
+gtd_panel_get_subtitle (GtdPanel *self)
+{
+  g_return_val_if_fail (GTD_IS_PANEL (self), NULL);
+  g_return_val_if_fail (GTD_PANEL_GET_IFACE (self)->get_icon, NULL);
+
+  return GTD_PANEL_GET_IFACE (self)->get_subtitle (self);
+}
