@@ -40,6 +40,12 @@ struct _GtdPanelInterface
   GList*             (*get_header_widgets)                 (GtdPanel        *panel);
 
   const GMenu*       (*get_menu)                           (GtdPanel        *panel);
+
+  GIcon*            (*get_icon)                            (GtdPanel        *self);
+
+  guint32           (*get_priority)                        (GtdPanel        *self);
+
+  gchar*            (*get_subtitle)                        (GtdPanel        *self);
 };
 
 const gchar*         gtd_panel_get_panel_name                    (GtdPanel           *panel);
@@ -50,6 +56,12 @@ GList*               gtd_panel_get_header_widgets                (GtdPanel
 
 const GMenu*         gtd_panel_get_menu                          (GtdPanel           *panel);
 
+GIcon*               gtd_panel_get_icon                          (GtdPanel           *self);
+
+guint32              gtd_panel_get_priority                      (GtdPanel           *self);
+
+gchar*               gtd_panel_get_subtitle                      (GtdPanel           *self);
+
 G_END_DECLS
 
 #endif /* GTD_PANEL_H */
