@@ -21,6 +21,7 @@
 #include "gtd-provider.h"
 #include "gtd-task.h"
 #include "gtd-task-list.h"
+#include "gtd-utils.h"
 
 /**
  * SECTION:gtd-provider
@@ -101,6 +102,18 @@ gtd_provider_default_init (GtdProviderInterface *iface)
                                                             NULL,
                                                             G_PARAM_READABLE));
 
+  /**
+   * GtdProvider::provider-type:
+   *
+   * The type of the #GtdProvider.
+   */
+  g_object_interface_install_property (iface,
+                                       g_param_spec_string ("provider-type",
+                                                            "Type of the provider",
+                                                            "The type of the provider",
+                                                            NULL,
+                                                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GtdProvider::description:
    *
@@ -218,6 +231,26 @@ gtd_provider_get_name (GtdProvider *provider)
   return GTD_PROVIDER_GET_IFACE (provider)->get_name (provider);
 }
 
+/**
+ * gtd_provider_get_provider_type:
+ * @provider: a #GtdProvider
+ *
+ * Retrieves the type of the @provider. This should return the
+ * same value, regardless of the account name.
+ *
+ * For example: "todoist", "todo-txt" or "google"
+ *
+ * Returns: (transfer none): the type of the @provider
+ */
+const gchar*
+gtd_provider_get_provider_type (GtdProvider *provider)
+{
+  g_return_val_if_fail (GTD_IS_PROVIDER (provider), NULL);
+  g_return_val_if_fail (GTD_PROVIDER_GET_IFACE (provider)->get_name, NULL);
+
+  return GTD_PROVIDER_GET_IFACE (provider)->get_provider_type (provider);
+}
+
 /**
  * gtd_provider_get_description:
  * @provider: a #GtdProvider
@@ -427,3 +460,34 @@ gtd_provider_set_default_task_list (GtdProvider *provider,
 
   return GTD_PROVIDER_GET_IFACE (provider)->set_default_task_list (provider, list);
 }
+
+/**
+ * gtd_provider_compare:
+ * @a: a #GtdProvider
+ * @b: a #GtdProvider
+ *
+ * Compares @a and @b. The sorting criteria is internal and
+ * may change.
+ *
+ * Returns: -1 if @a comes before @b, 1 for the oposite, and
+ * 0 if they're equal
+ */
+gint
+gtd_provider_compare (GtdProvider *a,
+                      GtdProvider *b)
+{
+  gint result;
+
+  g_return_val_if_fail (GTD_IS_PROVIDER (a), 0);
+  g_return_val_if_fail (GTD_IS_PROVIDER (b), 0);
+
+  if (a == b)
+    return 0;
+
+  result = gtd_collate_compare_strings (gtd_provider_get_name (a), gtd_provider_get_name (b));
+
+  if (result != 0)
+    return result;
+
+  return gtd_collate_compare_strings (gtd_provider_get_description (a), gtd_provider_get_description (b));
+}
