@@ -40,6 +40,8 @@ struct _GtdProviderInterface
 
   const gchar*       (*get_name)                                 (GtdProvider        *provider);
 
+  const gchar*       (*get_provider_type)                        (GtdProvider        *provider);
+
   const gchar*       (*get_description)                          (GtdProvider        *provider);
 
   gboolean           (*get_enabled)                              (GtdProvider        *provider);
@@ -81,6 +83,8 @@ const gchar*         gtd_provider_get_id                         (GtdProvider
 
 const gchar*         gtd_provider_get_name                       (GtdProvider        *provider);
 
+const gchar*         gtd_provider_get_provider_type              (GtdProvider        *provider);
+
 const gchar*         gtd_provider_get_description                (GtdProvider        *provider);
 
 gboolean             gtd_provider_get_enabled                    (GtdProvider        *provider);
@@ -114,6 +118,9 @@ GtdTaskList*         gtd_provider_get_default_task_list          (GtdProvider
 void                 gtd_provider_set_default_task_list          (GtdProvider        *provider,
                                                                   GtdTaskList        *list);
 
+gint                 gtd_provider_compare                        (GtdProvider        *a,
+                                                                  GtdProvider        *b);
+
 G_END_DECLS
 
 #endif /* GTD_PROVIDER_H */
